﻿using System;
using System.Text;

using RabbitMQ.Client;
using RabbitMQ.Client.Events;

namespace TestConsumer
{
    /// <summary>
    /// 프로그램
    /// </summary>
    class Program
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region 프로그램 시작하기 - Main(argumentArray)

        /// <summary>
        /// 프로그램 시작하기
        /// </summary>
        /// <param name="argumentArray">인자 배열</param>
        private static void Main(string[] argumentArray)
        {
            ConnectionFactory factory = new ConnectionFactory() { HostName = "localhost" };

            using(IConnection connection = factory.CreateConnection())
            {
                using(IModel model = connection.CreateModel())
                {
                    model.ExchangeDeclare
                    (
                        exchange : "direct_logs",
                        type     : "direct"
                    );

                    string queueName = model.QueueDeclare().QueueName;

                    if(argumentArray.Length < 1)
                    {
                        Console.WriteLine("사용법 : {0} [info] [warning] [errpr]", Environment.GetCommandLineArgs()[0]);

                        Console.WriteLine("프로그램을 종료하기 위해 아무 키나 눌러 주시기 바랍니다.");

                        Console.ReadKey(true);

                        Environment.ExitCode = 1;

                        return;
                    }

                    foreach(string argument in argumentArray)
                    {
                        model.QueueBind
                        (
                            queue      : queueName,
                            exchange   : "direct_logs",
                            routingKey : argument
                        );
                    }

                    WriteLog("메시지 대기...");

                    EventingBasicConsumer consumer = new EventingBasicConsumer(model);

                    consumer.Received += (sender, e) =>
                    {
                        string routingKey = e.RoutingKey;

                        byte[] messageArray = e.Body;

                        string message = Encoding.UTF8.GetString(messageArray);

                        WriteLog("수신 : {0}, {1}", routingKey, message);
                    };

                    model.BasicConsume
                    (
                        queue    : queueName,
                        autoAck  : true,
                        consumer : consumer
                    );

                    Console.WriteLine("프로그램을 종료하기 위해 아무 키나 눌러 주시기 바랍니다.");

                    Console.ReadKey(true);
                }
            }
        }

        #endregion
        #region 로그 작성하기 - WriteLog(format, parameterArray)

        /// <summary>
        /// 로그 작성하기
        /// </summary>
        /// <param name="format">포맷 문자열</param>
        /// <param name="parameterArray">매개 변수 배열</param>
        private static void WriteLog(string format, params object[] parameterArray)
        {
            string message;

            if(parameterArray.Length == 0)
            {
                message = format;
            }
            else
            {
                message = string.Format(format, parameterArray);
            }

            string log = string.Format("[{0}] {1}", DateTime.Now.ToString("HH:mm:ss"), message);

            Console.WriteLine(log);
        }

        #endregion
    }
}