﻿using System;
using System.Text;

using RabbitMQ.Client;

namespace TestProducer
{
    /// <summary>
    /// 프로그램
    /// </summary>
    class Program
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region 프로그램 시작하기 - Main()

        /// <summary>
        /// 프로그램 시작하기
        /// </summary>
        /// <param name="argumentArray">인자 배열</param>
        private static void Main(string[] argumentArray)
        {
            ConnectionFactory factory = new ConnectionFactory() { HostName = "localhost" };

            using(IConnection connection = factory.CreateConnection())
            {
                using(IModel model = connection.CreateModel())
                {
                    model.ExchangeDeclare
                    (
                        exchange : "direct_logs",
                        type     : "direct"
                    );

                    SendMessage(model, "error", "에러가 발생했습니다.");

                    SendMessage(model, "info", "알림 사항이 발생했습니다.");

                    SendMessage(model, "warning", "경고 사항이 발생했습니다.");
                }
            }

            Console.WriteLine("프로그램을 종료하려면 아무 키나 눌러 주시기 바랍니다.");

            Console.ReadKey(true);
        }

        #endregion

        #region 로그 작성하기 - WriteLog(format, parameterArray)

        /// <summary>
        /// 로그 작성하기
        /// </summary>
        /// <param name="format">포맷 문자열</param>
        /// <param name="parameterArray">매개 변수 배열</param>
        private static void WriteLog(string format, params object[] parameterArray)
        {
            string message;

            if(parameterArray.Length == 0)
            {
                message = format;
            }
            else
            {
                message = string.Format(format, parameterArray);
            }

            string log = string.Format("[{0}] {1}", DateTime.Now.ToString("HH:mm:ss"), message);

            Console.WriteLine(log);
        }

        #endregion
        #region 메시지 보내기 - SendMessage(model, routingKey, message)

        /// <summary>
        /// 메시지 보내기
        /// </summary>
        /// <param name="model">모델</param>
        /// <param name="routingKey">라우팅 키</param>
        /// <param name="message">메시지</param>
        private static void SendMessage(IModel model, string routingKey, string message)
        {
            byte[] messageArray = Encoding.UTF8.GetBytes(message);

            model.BasicPublish
            (
                exchange        : "direct_logs",
                routingKey      : routingKey,
                basicProperties : null,
                body            : messageArray
            );

            WriteLog("송신 : {0}, {1}", routingKey, message);
        }

        #endregion
    }
}